@@ -10,23 +10,25 @@ import pathlib
 import subprocess
 import collections
 import importlib.util
-import awyes.deploy
 
-NEW_LINE = "\n"
-USER_ENV_PATH = "*.env*"
-USER_PIPFILE_PATH = "Pipfile"
-USER_CONFIG_PATH_PREFIXES = "awyes.*"
-USER_CONFIG_PATH_SUFFIXES = (".yaml", ".yml")
-USER_CLIENT_PATH_PREFIXES = "awyes.*"
-USER_CLIENT_PATH_SUFFIXES = (".py",)
-USER_CLIENT_MODULE_NAME = "local"
+from .deploy import Deployment
+from .constants import (
+    NEW_LINE,
+    USER_ENV_PATH,
+    USER_PIPFILE_PATH,
+    USER_CLIENT_MODULE_NAME,
+    USER_CLIENT_PATH_PREFIXES,
+    USER_CLIENT_PATH_SUFFIXES,
+    USER_CONFIG_PATH_PREFIXES,
+    USER_CONFIG_PATH_SUFFIXES,
+)
 
 
 def load_config(path):
     collapse = lambda dl: dict(collections.ChainMap(*dl))
 
     return collapse(
-        collapse(yaml.safe_load_all(os.path.expandvars(p.read_text())))
+        collapse(reversed(list(yaml.safe_load_all(os.path.expandvars(p.read_text())))))
         for p in pathlib.Path(path).resolve().rglob(USER_CONFIG_PATH_PREFIXES)
         if p.suffix in USER_CONFIG_PATH_SUFFIXES
     )
@@ -107,7 +109,7 @@ def get_arguments():
         "--path",
         type=str,
         required=False,
-        default="Pipfile",
+        default=".",
         help="Path to user pipfile",
     )
     parser.add_argument(
@@ -136,10 +138,11 @@ def main():
     except:
         raise ValueError(f"couldn't find any clients at: {args.path}")
 
-    # Create and run the deployment
-    awyes.deploy.Deployment(args, config, clients).run(
-        get_actions(config, args.actions)
-    )
+    # Create and run the deployment if valid actions are found
+    if not (actions := get_actions(config, args.actions)):
+        raise ValueError(f"no actions found. Received: {args.actions}")
+
+    Deployment(args, config, clients).run(actions)
 
 
 if __name__ == "__main__":
